package fos

import scala.util.parsing.combinator.syntactical.StandardTokenParsers
import scala.util.parsing.input._

import Term._

/** This object implements a parser for the NB language of booleans and numbers found in Chapter 3 
 *  of the TAPL book. This object also serves as the `main` entry-point to the program.
 */
object Main extends StandardTokenParsers {
  lexical.reserved ++= List("true", "false", "if", "then", "else", "succ", "pred", "iszero")

  import lexical.NumericLit

  /** term ::= 'true'
             | 'false'
             | 'if' term 'then' term 'else' term
             | numericLit
             | 'succ' term
             | 'pred' term
             | 'iszero' term
   */
  def term: Parser[Term] =
      ( "true"  ^^^ True
      | "false" ^^^ False
      | "if" ~ term ~ "then" ~ term ~ "else" ~ term ^?
        { case "if" ~ e1 ~ "then" ~ e2 ~ "else" ~ e3 => If(e1, e2, e3) }
      | numericLit       ^^ { case chars => lit2Num(chars.toInt) }
      | "succ" ~> term   ^^ { case e1 => Succ(e1) }
      | "pred" ~> term   ^^ { case e1 => Pred(e1) }
      | "iszero" ~> term ^^ { case e1 => IsZero(e1) }
      | failure("illegal start of expression"))

  def lit2Num(n: Int): Term = if n == 0 then Zero else Succ(lit2Num(n - 1))

  def main(args: Array[String]): Unit =
    val stdin = new java.io.BufferedReader(new java.io.InputStreamReader(System.in))
    val tokens = new lexical.Scanner(stdin.readLine())
    phrase(term)(tokens) match
      case Success(trees, _) =>
        for (t <- Arithmetic.path(trees))
          println(t)
        try
          print("Big step: ")
          println(Arithmetic.eval(trees))
        catch
          case Arithmetic.TermIsStuck(t) => println("Stuck term: " + t)
      case e =>
        println(e)
  
}
